@@ -5,18 +5,20 @@ use crossterm::{
 };
 use frep_core::{replace::ReplaceResult, search::SearchResultWithReplacement};
 use futures::{Stream, StreamExt};
-use log::{error, LevelFilter};
+use log::{LevelFilter, error};
 use ratatui::{
+    Terminal,
     backend::{Backend, CrosstermBackend, TestBackend},
     crossterm::event::KeyEventKind,
-    Terminal,
 };
 use scooter_core::{
     app::{
         App, AppRunConfig, Event, EventHandlingResult, ExitAndReplaceState, ExitState, InputSource,
     },
+    config::{self, Config},
     errors::AppError,
     fields::SearchFieldValues,
+    keyboard::KeyEvent,
     replace::ReplaceState,
 };
 use std::{
@@ -30,12 +32,7 @@ use std::{
 };
 use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender};
 
-use crate::{
-    config::{load_config, Config},
-    conversions,
-    logging::DEFAULT_LOG_LEVEL,
-    tui::Tui,
-};
+use crate::{logging::DEFAULT_LOG_LEVEL, tui::Tui};
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 #[allow(clippy::struct_excessive_bools)]
@@ -69,7 +66,6 @@ pub type CrosstermEventStream = event::EventStream;
 
 pub struct AppRunner<B: Backend, E: EventStream, S: SnapshotProvider<B>> {
     app: App,
-    config: Config,
     event_receiver: UnboundedReceiver<Event>,
     tui: Tui<B>,
     event_stream: E,
@@ -126,7 +122,7 @@ impl AppRunner<CrosstermBackend<io::Stdout>, CrosstermEventStream, NoOpSnapshotP
         let backend = CrosstermBackend::new(io::stdout());
         let event_stream = CrosstermEventStream::new();
         let snapshot_provider = NoOpSnapshotProvider;
-        let user_config = load_config().context("Failed to read config file")?;
+        let user_config = config::load_config().context("Failed to read config file")?;
         Self::new(
             app_config,
             user_config,
@@ -140,23 +136,35 @@ impl AppRunner<CrosstermBackend<io::Stdout>, CrosstermEventStream, NoOpSnapshotP
 impl<E: EventStream> AppRunner<TestBackend, E, TestSnapshotProvider> {
     // Used in integration tests
     #[allow(dead_code)]
-    pub fn new_test_with_snapshot(
+    pub fn new_snapshot_test(
         app_config: AppConfig<'_>,
         backend: TestBackend,
         event_stream: E,
         snapshot_sender: UnboundedSender<String>,
     ) -> anyhow::Result<Self> {
-        let snapshot_provider = TestSnapshotProvider::new(snapshot_sender);
         // Tests should use default config, not load from user's config directory
         let test_config = Config::default();
-        Self::new(
+        Self::new_snapshot_test_override_config(
             app_config,
-            test_config,
             backend,
             event_stream,
-            snapshot_provider,
+            snapshot_sender,
+            test_config,
         )
     }
+
+    // Used in integration tests
+    #[allow(dead_code)]
+    pub fn new_snapshot_test_override_config(
+        app_config: AppConfig<'_>,
+        backend: TestBackend,
+        event_stream: E,
+        snapshot_sender: UnboundedSender<String>,
+        config: Config,
+    ) -> anyhow::Result<Self> {
+        let snapshot_provider = TestSnapshotProvider::new(snapshot_sender);
+        Self::new(app_config, config, backend, event_stream, snapshot_provider)
+    }
 }
 
 impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B, E, S> {
@@ -177,15 +185,14 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
             input_source,
             &app_config.search_field_values,
             &app_config.app_run_config,
-            config.search.disable_prepopulated_fields,
-        );
+            config,
+        )?;
 
         let terminal = Terminal::new(backend)?;
         let tui = Tui::new(terminal);
 
         Ok(Self {
             app,
-            config,
             event_receiver,
             tui,
             event_stream,
@@ -201,7 +208,7 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
     }
 
     pub fn draw(&mut self) -> anyhow::Result<()> {
-        self.tui.draw(&mut self.app, &self.config)?;
+        self.tui.draw(&mut self.app)?;
         self.snapshot_provider.send_snapshot(&self.tui);
         Ok(())
     }
@@ -212,11 +219,9 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
                 Some(Ok(event)) = self.event_stream.next() => {
                     match event {
                         CrosstermEvent::Key(key) if key.kind == KeyEventKind::Press => {
-                            if let Some((code, modifiers)) = conversions::convert_key_event(&key) {
-                                self.app.handle_key_event(code, modifiers)
-                            } else {
-                                EventHandlingResult::None
-                            }
+                            let mut key_event: KeyEvent = key.into();
+                            key_event.canonicalize();
+                            self.app.handle_key_event(key_event)
                         },
                         CrosstermEvent::Resize(_, _) => EventHandlingResult::Rerender,
                         _ => EventHandlingResult::None,
@@ -229,7 +234,7 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
                             self.tui.show_cursor()?;
                             match self.open_editor(file_path, line) {
                                 Ok(()) => {
-                                    if self.config.editor_open.exit {
+                                    if self.app.config.editor_open.exit {
                                         res = EventHandlingResult::Exit(None);
                                     }
                                 }
@@ -280,7 +285,7 @@ impl<B: Backend + 'static, E: EventStream, S: SnapshotProvider<B>> AppRunner<B,
     }
 
     fn open_editor(&self, file_path: PathBuf, line: usize) -> anyhow::Result<()> {
-        match &self.config.editor_open.command {
+        match &self.app.config.editor_open.command {
             Some(command) => {
                 Self::open_editor_from_command(command, &file_path, line)?;
             }
@@ -432,7 +437,15 @@ pub async fn run_app_tui(app_config: AppConfig<'_>) -> anyhow::Result<Option<Str
                 None
             }
         }
-        None => None,
+        None => {
+            if runner.app.print_on_exit {
+                match runner.app.input_source {
+                    InputSource::Stdin(stdin) => write!(io::stderr(), "{stdin}")?,
+                    InputSource::Directory(_) => unreachable!(),
+                }
+            }
+            None
+        }
     }
     .map(|stats| {
         format_replacement_results(
