@@ -10,16 +10,15 @@ use std::{
 
 use scooter_core::{
     app::AppRunConfig,
+    config,
     fields::{FieldValue, SearchFieldValues},
 };
 
-use app_runner::{run_app_tui, AppConfig};
+use app_runner::{AppConfig, run_app_tui};
 use headless::{run_headless, run_headless_with_stdin};
-use logging::{setup_logging, DEFAULT_LOG_LEVEL};
+use logging::{DEFAULT_LOG_LEVEL, setup_logging};
 
 mod app_runner;
-mod config;
-mod conversions;
 mod headless;
 mod logging;
 mod tui;
@@ -78,6 +77,10 @@ struct Args {
     #[arg(long)]
     no_stdin: bool,
 
+    /// Print stdin content on exit
+    #[arg(long)]
+    print_on_exit: bool,
+
     // --- Initial values for fields ---
     //
     /// Text to search with
@@ -138,7 +141,9 @@ fn validate_flag_combinations(args: &Args) -> anyhow::Result<()> {
     if args.immediate_search || args.immediate_replace || args.print_results {
         for (name, enabled) in [("--no-tui", args.no_tui), ("--immediate", args.immediate)] {
             if enabled {
-                bail!("{name} cannot be combined with --immediate-search, --immediate-replace, or --print-results");
+                bail!(
+                    "{name} cannot be combined with --immediate-search, --immediate-replace, or --print-results"
+                );
             }
         }
     }
@@ -185,6 +190,8 @@ fn validate_stdin_usage(args: &Args, stdin_content: Option<&str>) -> anyhow::Res
         if args.files_to_exclude.is_some() {
             bail!("Cannot use --files-to-exclude when processing stdin");
         }
+    } else if args.print_on_exit {
+        bail!("Cannot use --print-on-exit when not processing stdin");
     }
     Ok(())
 }
@@ -211,6 +218,7 @@ impl<'a> TryFrom<&'a Args> for AppConfig<'a> {
                 immediate_search: args.immediate_search || immediate,
                 immediate_replace: args.immediate_replace || immediate,
                 print_results: args.print_results || immediate,
+                print_on_exit: args.print_on_exit,
             },
             stdin_content,
         })
@@ -314,6 +322,7 @@ mod tests {
             immediate: false,
             no_tui: false,
             no_stdin: false,
+            print_on_exit: false,
             search_text: None,
             replace_text: None,
             fixed_strings: false,
@@ -340,10 +349,12 @@ mod tests {
         };
         let result = validate_flag_combinations(&args);
         assert!(result.is_err());
-        assert!(result
-            .unwrap_err()
-            .to_string()
-            .contains("--no-tui cannot be combined with --immediate"));
+        assert!(
+            result
+                .unwrap_err()
+                .to_string()
+                .contains("--no-tui cannot be combined with --immediate")
+        );
     }
 
     #[test]
@@ -381,10 +392,12 @@ mod tests {
                 result.is_err(),
                 "Expected error for --no-tui with {flag_name}"
             );
-            assert!(result
-                .unwrap_err()
-                .to_string()
-                .contains("--no-tui cannot be combined with"));
+            assert!(
+                result
+                    .unwrap_err()
+                    .to_string()
+                    .contains("--no-tui cannot be combined with")
+            );
         }
     }
 
@@ -423,10 +436,12 @@ mod tests {
                 result.is_err(),
                 "Expected error for --immediate with {flag_name}"
             );
-            assert!(result
-                .unwrap_err()
-                .to_string()
-                .contains("--immediate cannot be combined with"));
+            assert!(
+                result
+                    .unwrap_err()
+                    .to_string()
+                    .contains("--immediate cannot be combined with")
+            );
         }
     }
 
@@ -472,10 +487,12 @@ mod tests {
                 result.is_err(),
                 "Expected error for {flag_name} without search text"
             );
-            assert!(result
-                .unwrap_err()
-                .to_string()
-                .contains(&format!("{flag_name} requires --search-text")));
+            assert!(
+                result
+                    .unwrap_err()
+                    .to_string()
+                    .contains(&format!("{flag_name} requires --search-text"))
+            );
         }
     }
 
